@@ -32,6 +32,14 @@
  * consideration. See <ulink url="http://www.vorbis.com/">Ogg/Vorbis</ulink>
  * for a royalty free (and often higher quality) alternative.
  * </para>
+ * <title>Output sample rate</title>
+ * <para>
+ * If no fixed output sample rate is negotiated on the element's src pad,
+ * the element will choose an optimal sample rate to resample to internally.
+ * For example, a 16-bit 44.1 KHz mono audio stream encoded at 48 kbit will
+ * get resampled to 32 KHz.  Use filter caps on the src pad to force a
+ * particular sample rate.
+ * </para>
  * <title>Writing metadata (tags)</title>
  * <para>
  * Whilst the lame encoder element does claim to implement the GstTagSetter
@@ -65,9 +73,15 @@
  * <programlisting>
  * gst-launch -v cdda://5 ! audioconvert ! lame bitrate=192 ! filesink location=track5.mp3
  * </programlisting>
+ * <para>
+ * Encode to a fixed sample rate:
+ * </para>
+ * <programlisting>
+ * gst-launch -v audiotestsrc num-buffers=10 ! audio/x-raw-int,rate=44100,channels=1 ! lame bitrate=48 mode=3 ! filesink location=test.mp3
+ * </programlisting>
  * </refsect2>
  *
- * Last reviewed on 2006-10-13 (0.10.4)
+ * Last reviewed on 2007-07-24 (0.10.7)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -495,6 +509,13 @@ gst_lame_class_init (GstLameClass * klass)
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_lame_change_state);
 }
 
+static gboolean
+gst_lame_src_setcaps (GstPad * pad, GstCaps * caps)
+{
+  GST_DEBUG_OBJECT (pad, "caps: %" GST_PTR_FORMAT, caps);
+  return TRUE;
+}
+
 static gboolean
 gst_lame_sink_setcaps (GstPad * pad, GstCaps * caps)
 {
@@ -529,7 +550,7 @@ gst_lame_sink_setcaps (GstPad * pad, GstCaps * caps)
       gst_caps_new_simple ("audio/mpeg",
       "mpegversion", G_TYPE_INT, 1,
       "layer", G_TYPE_INT, 3,
-      "channels", G_TYPE_INT, lame->num_channels,
+      "channels", G_TYPE_INT, lame->mode == MONO ? 1 : lame->num_channels,
       "rate", G_TYPE_INT, out_samplerate, NULL);
 
   /* and use these caps */
@@ -579,6 +600,8 @@ gst_lame_init (GstLame * lame)
 
   lame->srcpad =
       gst_pad_new_from_static_template (&gst_lame_src_template, "src");
+  gst_pad_set_setcaps_function (lame->srcpad,
+      GST_DEBUG_FUNCPTR (gst_lame_src_setcaps));
   gst_element_add_pad (GST_ELEMENT (lame), lame->srcpad);
 
   /* create an encoder state so we can ask about defaults */
@@ -601,6 +624,7 @@ gst_lame_init (GstLame * lame)
   lame->quality = 5;            /* lame_get_quality (lame->lgf);
                                  * => -1/out of range */
   lame->mode = lame_get_mode (lame->lgf);
+  lame->requested_mode = lame->mode;
   lame->force_ms = lame_get_force_ms (lame->lgf);
   lame->free_format = lame_get_free_format (lame->lgf);
   lame->copyright = lame_get_copyright (lame->lgf);
@@ -718,7 +742,7 @@ gst_lame_set_property (GObject * object, guint prop_id, const GValue * value,
       lame->quality = g_value_get_enum (value);
       break;
     case ARG_MODE:
-      lame->mode = g_value_get_enum (value);
+      lame->requested_mode = g_value_get_enum (value);
       break;
     case ARG_FORCE_MS:
       lame->force_ms = g_value_get_boolean (value);
@@ -833,7 +857,7 @@ gst_lame_get_property (GObject * object, guint prop_id, GValue * value,
       g_value_set_enum (value, lame->quality);
       break;
     case ARG_MODE:
-      g_value_set_enum (value, lame->mode);
+      g_value_set_enum (value, lame->requested_mode);
       break;
     case ARG_FORCE_MS:
       g_value_set_boolean (value, lame->force_ms);
@@ -951,8 +975,8 @@ gst_lame_sink_event (GstPad * pad, GstEvent * event)
         if (size > 0 && lame->last_flow == GST_FLOW_OK) {
           gint64 duration;
 
-          duration = gst_util_uint64_scale_int (size, GST_SECOND,
-              2 * lame->samplerate * lame->num_channels);
+          duration = gst_util_uint64_scale (size, 8 * GST_SECOND,
+              1000 * lame->bitrate);
 
           if (lame->last_ts == GST_CLOCK_TIME_NONE) {
             lame->last_ts = lame->eos_ts;
@@ -1130,13 +1154,16 @@ init_error:
 static gboolean
 gst_lame_setup (GstLame * lame)
 {
+
 #define CHECK_ERROR(command) G_STMT_START {\
   if ((command) < 0) { \
     GST_ERROR_OBJECT (lame, "setup failed: " G_STRINGIFY (command)); \
     return FALSE; \
   } \
 }G_STMT_END
+
   int retval;
+  GstCaps *allowed_caps;
 
   GST_DEBUG_OBJECT (lame, "starting setup");
 
@@ -1153,15 +1180,38 @@ gst_lame_setup (GstLame * lame)
   if (lame->lgf == NULL)
     return FALSE;
 
-  /* let lame choose a default samplerate */
-  lame_set_out_samplerate (lame->lgf, 0);
-
   /* copy the parameters over */
   lame_set_in_samplerate (lame->lgf, lame->samplerate);
 
+  /* let lame choose default samplerate unless outgoing sample rate is fixed */
+  allowed_caps = gst_pad_get_allowed_caps (lame->srcpad);
+
+  if (allowed_caps != NULL) {
+    GstStructure *structure;
+    gint samplerate;
+
+    structure = gst_caps_get_structure (allowed_caps, 0);
+
+    if (gst_structure_get_int (structure, "rate", &samplerate)) {
+      GST_DEBUG_OBJECT (lame, "Setting sample rate to %d as fixed in src caps",
+          samplerate);
+      lame_set_out_samplerate (lame->lgf, samplerate);
+    } else {
+      GST_DEBUG_OBJECT (lame, "Letting lame choose sample rate");
+      lame_set_out_samplerate (lame->lgf, 0);
+    }
+    gst_caps_unref (allowed_caps);
+    allowed_caps = NULL;
+  } else {
+    GST_DEBUG_OBJECT (lame, "No peer yet, letting lame choose sample rate");
+    lame_set_out_samplerate (lame->lgf, 0);
+  }
+
   /* force mono encoding if we only have one channel */
   if (lame->num_channels == 1)
     lame->mode = 3;
+  else
+    lame->mode = lame->requested_mode;
 
   CHECK_ERROR (lame_set_num_channels (lame->lgf, lame->num_channels));
   CHECK_AND_FIXUP_BITRATE (lame, "bitrate", lame->bitrate, lame->free_format);
