@@ -18,6 +18,35 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-siddec
+ * @short_description: a decoder for sid files
+ *
+ * <refsect2>
+ * <para>
+ * This element decodes .sid files to raw audio. .sid files are in fact 
+ * small Commodore 64 programs that are executed on an emulated 6502 CPU and a 
+ * MOS 6581 sound chip. 
+ * </para>
+ * <para>
+ * This plugin will first load the complete program into memory before starting
+ * the emulator and producing output.
+ * </para>
+ * <para>
+ * Seeking is not (and cannot be) implemented.
+ * </para>
+ * <title>Example pipelines</title>
+ * <para>
+ * <programlisting>
+ * gst-launch -v filesrc location=Hawkeye.sid ! siddec ! audioconvert ! alsasink
+ * </programlisting>
+ * Decode a sid file and play back the audio using alsasink.
+ * </para>
+ * </refsect2>
+ *
+ * Last reviewed on 2006-12-30 (0.10.5)
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -25,19 +54,6 @@
 #include <string.h>
 #include "gstsiddec.h"
 
-static const GstElementDetails gst_siddec_details =
-GST_ELEMENT_DETAILS ("Sid decoder",
-    "Codec/Decoder/Audio",
-    "Use libsidplay to decode SID audio tunes",
-    "Wim Taymans <wim@fluendo.com> ");
-
-/* Sidec signals and args */
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
-
 #define DEFAULT_TUNE		0
 #define DEFAULT_CLOCK		SIDTUNE_CLOCK_PAL
 #define DEFAULT_MEMORY		MPU_BANK_SWITCHING
@@ -134,8 +150,6 @@ static void gst_siddec_set_property (GObject * object, guint prop_id,
 
 static GstElementClass *parent_class = NULL;
 
-//static guint gst_siddec_signals[LAST_SIGNAL] = { 0 };
-
 GType
 gst_siddec_get_type (void)
 {
@@ -168,7 +182,9 @@ gst_siddec_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details (element_class, &gst_siddec_details);
+  gst_element_class_set_details_simple (element_class, "Sid decoder",
+      "Codec/Decoder/Audio", "Use libsidplay to decode SID audio tunes",
+      "Wim Taymans <wim.taymans@gmail.com>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_templ));
@@ -225,17 +241,13 @@ gst_siddec_class_init (GstSidDec * klass)
 static void
 gst_siddec_init (GstSidDec * siddec)
 {
-  siddec->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get (&sink_templ),
-      "sink");
+  siddec->sinkpad = gst_pad_new_from_static_template (&sink_templ, "sink");
   gst_pad_set_query_function (siddec->sinkpad, NULL);
   gst_pad_set_event_function (siddec->sinkpad, gst_siddec_sink_event);
   gst_pad_set_chain_function (siddec->sinkpad, gst_siddec_chain);
   gst_element_add_pad (GST_ELEMENT (siddec), siddec->sinkpad);
 
-  siddec->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get (&src_templ),
-      "src");
+  siddec->srcpad = gst_pad_new_from_static_template (&src_templ, "src");
   gst_pad_set_event_function (siddec->srcpad, gst_siddec_src_event);
   gst_pad_set_query_function (siddec->srcpad, gst_siddec_src_query);
   gst_pad_use_fixed_caps (siddec->srcpad);
@@ -274,7 +286,6 @@ gst_siddec_finalize (GObject * object)
   delete (siddec->engine);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
-
 }
 
 static void
@@ -356,6 +367,7 @@ siddec_negotiate (GstSidDec * siddec)
 
   return TRUE;
 
+  /* ERRORS */
 nothing_allowed:
   {
     GST_DEBUG_OBJECT (siddec, "could not get allowed caps");
@@ -539,7 +551,7 @@ gst_siddec_chain (GstPad * pad, GstBuffer * buffer)
 overflow:
   {
     GST_ELEMENT_ERROR (siddec, STREAM, DECODE,
-        (NULL), ("Input data bigger than allowd buffer size"));
+        (NULL), ("Input data bigger than allowed buffer size"));
     gst_object_unref (siddec);
     return GST_FLOW_ERROR;
   }
@@ -764,8 +776,9 @@ plugin_init (GstPlugin * plugin)
       GST_TYPE_SIDDEC);
 }
 
+/* FIXME: remove cast to gchar once we depend on core >= 0.10.14.1 */
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "siddec",
-    "Uses libsidplay to decode .sid files",
+    (gchar *) "Uses libsidplay to decode .sid files",
     plugin_init, VERSION, "GPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
