@@ -25,6 +25,7 @@
 #endif
 
 #include "gstdvdsubdec.h"
+#include "gstdvdsubparse.h"
 #include <string.h>
 
 GST_BOILERPLATE (GstDvdSubDec, gst_dvd_sub_dec, GstElement, GST_TYPE_ELEMENT);
@@ -43,13 +44,6 @@ static gboolean gst_dvd_sub_dec_sink_event (GstPad * pad, GstEvent * event);
 static GstFlowReturn gst_send_subtitle_frame (GstDvdSubDec * dec,
     GstClockTime end_ts);
 
-static const GstElementDetails gst_dvd_sub_dec_details =
-GST_ELEMENT_DETAILS ("DVD subtitle Decoder",
-    "Codec/Decoder/Video",
-    "Decodes DVD subtitles into AYUV video frames",
-    "Wim Taymans <wim.taymans@chello.be>, "
-    "Jan Schmidt <thaytan@mad.scientist.com>");
-
 static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -110,7 +104,10 @@ gst_dvd_sub_dec_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&subtitle_template));
 
-  gst_element_class_set_details (element_class, &gst_dvd_sub_dec_details);
+  gst_element_class_set_details_simple (element_class, "DVD subtitle decoder",
+      "Codec/Decoder/Video", "Decodes DVD subtitles into AYUV video frames",
+      "Wim Taymans <wim.taymans@gmail.com>, "
+      "Jan Schmidt <thaytan@mad.scientist.com>");
 }
 
 static void
@@ -130,9 +127,7 @@ gst_dvd_sub_dec_init (GstDvdSubDec * dec, GstDvdSubDecClass * klass)
 {
   GstPadTemplate *tmpl;
 
-  dec->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&subtitle_template), "sink");
+  dec->sinkpad = gst_pad_new_from_static_template (&subtitle_template, "sink");
   gst_pad_set_chain_function (dec->sinkpad,
       GST_DEBUG_FUNCPTR (gst_dvd_sub_dec_chain));
   gst_pad_set_event_function (dec->sinkpad,
@@ -243,7 +238,7 @@ gst_dvd_sub_dec_parse_subpic (GstDvdSubDec * dec)
   gboolean broken = FALSE;
   gboolean last_seq = FALSE;
   guchar *next_seq = NULL;
-  guint event_time;
+  GstClockTime event_time;
 
   /* nothing to do if we finished this buffer already */
   if (dec->parse_pos == NULL)
@@ -368,13 +363,13 @@ gst_dvd_sub_dec_parse_subpic (GstDvdSubDec * dec)
 
         /* Start a new control sequence */
         if (buf + 4 < end) {
-          gint ticks = GST_READ_UINT16_BE (buf);
+          guint16 ticks = GST_READ_UINT16_BE (buf);
 
           event_time = gst_util_uint64_scale (ticks, 1024 * GST_SECOND, 90000);
 
           GST_DEBUG_OBJECT (dec,
               "Next DCSQ at offset %d, delay %g secs (%d ticks)", buf - start,
-              (gdouble) event_time / GST_SECOND, ticks);
+              gst_util_guint64_to_gdouble (event_time / GST_SECOND), ticks);
 
           dec->parse_pos = buf;
           if (event_time > 0) {
@@ -870,7 +865,7 @@ gst_dvd_sub_dec_sink_event (GstPad * pad, GstEvent * event)
       }
       break;
     }
-    case GST_EVENT_FLUSH_START:{
+    case GST_EVENT_FLUSH_STOP:{
       /* Turn off forced highlight display */
       dec->forced_display = 0;
       dec->current_button = 0;
@@ -991,12 +986,14 @@ static gboolean
 plugin_init (GstPlugin * plugin)
 {
   if (!gst_element_register (plugin, "dvdsubdec", GST_RANK_NONE,
-          GST_TYPE_DVD_SUB_DEC)) {
+          GST_TYPE_DVD_SUB_DEC) ||
+      !gst_element_register (plugin, "dvdsubparse", GST_RANK_NONE,
+          GST_TYPE_DVD_SUB_PARSE)) {
     return FALSE;
   }
 
   GST_DEBUG_CATEGORY_INIT (gst_dvd_sub_dec_debug, "dvdsubdec", 0,
-      "DVD subtitle decoder element");
+      "DVD subtitle decoder");
 
   return TRUE;
 }
@@ -1004,5 +1001,5 @@ plugin_init (GstPlugin * plugin)
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "dvdsub",
-    "Decode DVD subtitles to AYUV video frames", plugin_init,
-    VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
+    "DVD subtitle parser and decoder", plugin_init,
+    VERSION, GST_LICENSE, GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
