@@ -177,15 +177,12 @@ static void
 ac3iec_init (AC3IEC * ac3iec)
 {
   ac3iec->sink =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&ac3iec_sink_template), "sink");
+      gst_pad_new_from_static_template (&ac3iec_sink_template, "sink");
   gst_pad_set_setcaps_function (ac3iec->sink, ac3iec_setcaps);
   gst_pad_set_chain_function (ac3iec->sink, ac3iec_chain_dvd);
   gst_element_add_pad (GST_ELEMENT (ac3iec), ac3iec->sink);
 
-  ac3iec->src =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&ac3iec_src_template), "src");
+  ac3iec->src = gst_pad_new_from_static_template (&ac3iec_src_template, "src");
   gst_pad_use_fixed_caps (ac3iec->src);
   gst_element_add_pad (GST_ELEMENT (ac3iec), ac3iec->src);
 
@@ -201,6 +198,8 @@ ac3iec_finalize (GObject * object)
   AC3IEC *ac3iec = AC3IEC (object);
 
   g_free (ac3iec->padder);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -437,8 +436,10 @@ ac3iec_change_state (GstElement * element, GstStateChange transition)
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       ac3p_clear (ac3iec->padder);
-      gst_caps_unref (ac3iec->caps);
-      ac3iec->caps = NULL;
+      if (ac3iec->caps) {
+        gst_caps_unref (ac3iec->caps);
+        ac3iec->caps = NULL;
+      }
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
