@@ -139,8 +139,6 @@ static void gst_dvd_demux_base_init (gpointer klass);
 static void gst_dvd_demux_init (GstDVDDemux * dvd_demux,
     GstDVDDemuxClass * klass);
 
-static GstFlowReturn gst_dvd_demux_send_buffer (GstMPEGParse * mpeg_parse,
-    GstBuffer * buffer, GstClockTime time);
 static gboolean gst_dvd_demux_process_event (GstMPEGParse * mpeg_parse,
     GstEvent * event);
 
@@ -167,6 +165,9 @@ static GstFlowReturn gst_dvd_demux_send_subbuffer
     GstMPEGStream * outstream,
     GstBuffer * buffer, GstClockTime timestamp, guint offset, guint size);
 
+static GstFlowReturn gst_dvd_demux_combine_flows (GstMPEGDemux * mpegdemux,
+    GstMPEGStream * stream, GstFlowReturn flow);
+
 static void gst_dvd_demux_set_cur_audio
     (GstDVDDemux * dvd_demux, gint stream_nr);
 static void gst_dvd_demux_set_cur_subpicture
@@ -192,7 +193,7 @@ gst_dvd_demux_base_init (gpointer klass)
   GstMPEGDemuxClass *demux_class = GST_MPEG_DEMUX_CLASS (klass);
   GstDVDDemuxClass *dvd_demux_class = GST_DVD_DEMUX_CLASS (klass);
 
-  mpeg_parse_class->send_buffer = gst_dvd_demux_send_buffer;
+  mpeg_parse_class->send_buffer = NULL;
   mpeg_parse_class->process_event = gst_dvd_demux_process_event;
 
   /* sink pad */
@@ -245,6 +246,7 @@ gst_dvd_demux_class_init (GstDVDDemuxClass * klass)
   mpeg_demux_class->get_audio_stream = gst_dvd_demux_get_audio_stream;
   mpeg_demux_class->get_video_stream = gst_dvd_demux_get_video_stream;
   mpeg_demux_class->send_subbuffer = gst_dvd_demux_send_subbuffer;
+  mpeg_demux_class->combine_flows = gst_dvd_demux_combine_flows;
   mpeg_demux_class->process_private = gst_dvd_demux_process_private;
   mpeg_demux_class->synchronise_pads = gst_dvd_demux_synchronise_pads;
   mpeg_demux_class->sync_stream_to_time = gst_dvd_demux_sync_stream_to_time;
@@ -288,15 +290,6 @@ gst_dvd_demux_init (GstDVDDemux * dvd_demux, GstDVDDemuxClass * klass)
   dvd_demux->langcodes = NULL;
 }
 
-
-static GstFlowReturn
-gst_dvd_demux_send_buffer (GstMPEGParse * mpeg_parse, GstBuffer * buffer,
-    GstClockTime time)
-{
-  gst_buffer_unref (buffer);
-  return GST_FLOW_OK;
-}
-
 static gboolean
 gst_dvd_demux_process_event (GstMPEGParse * mpeg_parse, GstEvent * event)
 {
@@ -326,6 +319,10 @@ gst_dvd_demux_process_event (GstMPEGParse * mpeg_parse, GstEvent * event)
       dvd_demux->segment_filter = TRUE;
       ret = GST_MPEG_PARSE_CLASS (parent_class)->process_event (mpeg_parse,
           event);
+
+      /* parent class will have reset the other streams */
+      gst_mpeg_streams_reset_last_flow (dvd_demux->subpicture_stream,
+          GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS);
       break;
     case GST_EVENT_CUSTOM_DOWNSTREAM:
     case GST_EVENT_CUSTOM_DOWNSTREAM_OOB:
@@ -448,8 +445,8 @@ gst_dvd_demux_handle_dvd_event (GstDVDDemux * dvd_demux, GstEvent * event)
     gst_element_no_more_pads (GST_ELEMENT (dvd_demux));
 
     /* Keep video/audio/subtitle pads within 1/2 sec of the SCR */
-    mpeg_demux->max_gap = 0.5 * GST_SECOND;
-    mpeg_demux->max_gap_tolerance = 0.05 * GST_SECOND;
+    mpeg_demux->max_gap = GST_SECOND / 2;
+    mpeg_demux->max_gap_tolerance = GST_SECOND / 20;
   } else {
     GST_DEBUG_OBJECT (dvd_demux, "dvddemux Forwarding DVD event %s to all pads",
         event_type);
@@ -654,6 +651,7 @@ gst_dvd_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
       GST_DEBUG_OBJECT (mpeg_demux, "adding pad %s with language = %s",
           GST_PAD_NAME (str->pad), (lang_code) ? lang_code : "(unknown)");
 
+      gst_pad_set_active (str->pad, TRUE);
       gst_element_add_pad (GST_ELEMENT (mpeg_demux), str->pad);
 
       if (codec || lang_code) {
@@ -735,6 +733,7 @@ gst_dvd_demux_get_subpicture_stream (GstMPEGDemux * mpeg_demux,
     }
 
     if (add_pad) {
+      gst_pad_set_active (str->pad, TRUE);
       gst_element_add_pad (GST_ELEMENT (mpeg_demux), str->pad);
 
       if (dvd_demux->langcodes) {
@@ -916,6 +915,60 @@ gst_dvd_demux_process_private (GstMPEGDemux * mpeg_demux,
   return ret;
 }
 
+/* random magic value */
+#define MIN_BUFS_FOR_NO_MORE_PADS 100
+
+static GstFlowReturn
+gst_dvd_demux_combine_flows (GstMPEGDemux * mpegdemux, GstMPEGStream * stream,
+    GstFlowReturn flow)
+{
+  GstDVDDemux *demux = (GstDVDDemux *) mpegdemux;
+  gint i;
+
+  /* store the value */
+  stream->last_flow = flow;
+
+  /* if it's success we can return the value right away */
+  if (GST_FLOW_IS_SUCCESS (flow))
+    goto done;
+
+  /* any other error that is not-linked can be returned right
+   * away */
+  if (flow != GST_FLOW_NOT_LINKED) {
+    GST_DEBUG_OBJECT (demux, "flow %s on pad %" GST_PTR_FORMAT,
+        gst_flow_get_name (flow), stream->pad);
+    goto done;
+  }
+
+  /* let parent class check for anything non-not-linked for its streams */
+  flow =
+      GST_MPEG_DEMUX_CLASS (parent_class)->combine_flows (mpegdemux, stream,
+      flow);
+  if (flow != GST_FLOW_NOT_LINKED)
+    goto done;
+
+  /* only return NOT_LINKED if all other pads returned NOT_LINKED */
+  for (i = 0; i < GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS; i++) {
+    if (demux->subpicture_stream[i] != NULL) {
+      flow = demux->subpicture_stream[i]->last_flow;
+      /* some other return value (must be SUCCESS but we can return
+       * other values as well) */
+      if (flow != GST_FLOW_NOT_LINKED)
+        goto done;
+      if (demux->subpicture_stream[i]->buffers_sent < MIN_BUFS_FOR_NO_MORE_PADS) {
+        flow = GST_FLOW_OK;
+        goto done;
+      }
+    }
+  }
+  /* if we get here, all other pads were unlinked and we return
+   * NOT_LINKED then */
+  GST_DEBUG_OBJECT (demux, "all pads combined have not-linked flow");
+
+done:
+  return flow;
+}
+
 static GstFlowReturn
 gst_dvd_demux_send_subbuffer (GstMPEGDemux * mpeg_demux,
     GstMPEGStream * outstream, GstBuffer * buffer,
@@ -985,10 +1038,22 @@ gst_dvd_demux_send_subbuffer (GstMPEGDemux * mpeg_demux,
     gst_buffer_set_caps (outbuf, outstream->caps);
 
     ret = gst_pad_push (outpad, outbuf);
+
+    /* this was one of the current_foo pads, which is shadowing one of the
+     * foo_%02d pads, so since we keep track of the last flow value in the
+     * stream structure we need to combine the OK/NOT_LINKED flows here
+     * (because both pads share the same stream structure) */
+    if ((ret == GST_FLOW_NOT_LINKED && outstream->last_flow == GST_FLOW_OK) ||
+        (ret == GST_FLOW_OK && outstream->last_flow == GST_FLOW_NOT_LINKED)) {
+      outstream->last_flow = GST_FLOW_OK;
+      ret = GST_FLOW_OK;
+    }
   }
 
   gst_buffer_unref (buffer);
 
+  ret = DEMUX_CLASS (dvd_demux)->combine_flows (mpeg_demux, outstream, ret);
+
   return ret;
 }
 
@@ -1052,7 +1117,7 @@ gst_dvd_demux_reset (GstDVDDemux * dvd_demux)
 {
   int i;
 
-  //GstMPEGDemux *mpeg_demux = GST_MPEG_DEMUX (dvd_demux);
+  GstMPEGDemux *mpeg_demux = GST_MPEG_DEMUX (dvd_demux);
 
   GST_INFO ("Resetting the dvd demuxer");
   for (i = 0; i < GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS; i++) {
@@ -1075,11 +1140,9 @@ gst_dvd_demux_reset (GstDVDDemux * dvd_demux)
   dvd_demux->cur_subpicture_nr = 0;
   dvd_demux->mpeg_version = 0;
 
-#if 0
   /* Reset max_gap handling */
-  mpeg_demux->max_gap = GST_CLOCK_TIME_NONE;
-  mpeg_demux->max_gap_tolerance = GST_CLOCK_TIME_NONE;
-#endif
+  mpeg_demux->max_gap = 0.5 * GST_SECOND;
+  mpeg_demux->max_gap_tolerance = 0.05 * GST_SECOND;
 }
 
 static void
@@ -1106,14 +1169,9 @@ gst_dvd_demux_sync_stream_to_time (GstMPEGDemux * mpeg_demux,
     GstMPEGStream * stream, GstClockTime last_ts)
 {
   GstDVDDemux *dvd_demux = GST_DVD_DEMUX (mpeg_demux);
-
-#if 0
-  GstClockTime start_ts;
-  GstEvent *filler = NULL;
-  GstFormat fmt = GST_FORMAT_TIME;
-#endif
-  GstPad *outpad = NULL;
-  gint cur_nr = 0;
+  GstMPEGParse *mpeg_parse = GST_MPEG_PARSE (mpeg_demux);
+  GstPad *outpad;
+  gint cur_nr;
 
   parent_class->sync_stream_to_time (mpeg_demux, stream, last_ts);
 
@@ -1130,27 +1188,15 @@ gst_dvd_demux_sync_stream_to_time (GstMPEGDemux * mpeg_demux,
       outpad = dvd_demux->cur_subpicture;
       cur_nr = dvd_demux->cur_subpicture_nr;
       break;
+    default:
+      return;
   }
 
-#if 0
-  /* FIXME: fillers in 0.9 aren't specified properly yet */
-  if ((outpad != NULL) && (cur_nr == stream->number)) {
-    if (GST_PAD_PEER (stream->pad)
-        && gst_pad_query_position (GST_PAD_PEER (stream->pad), &fmt,
-            (gint64 *) & start_ts)) {
-      if (start_ts < last_ts)
-        filler =
-            gst_event_new_filler_stamped (start_ts, GST_CLOCK_DIFF (last_ts,
-                start_ts));
-    } else
-      filler = gst_event_new_filler_stamped (last_ts, GST_CLOCK_TIME_NONE);
-
-    if (filler) {
-      if (!gst_pad_push_event (stream->pad, filler))
-        gst_event_unref (filler);
-    }
+  if (outpad && (cur_nr == stream->number)) {
+    gst_pad_push_event (outpad, gst_event_new_new_segment (TRUE,
+            mpeg_parse->current_segment.rate, GST_FORMAT_TIME,
+            last_ts, -1, last_ts));
   }
-#endif
 }
 
 static GstStateChangeReturn
