@@ -1,4 +1,4 @@
-/* GStreamer
+/* GStreamer RealMedia demuxer
  * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
  * Copyright (C) <2003> David A. Schleef <ds@schleef.org>
  * Copyright (C) <2004> Stephane Loeuillet <gstreamer@leroutier.net>
@@ -6,6 +6,7 @@
  * Copyright (C) <2005> Michael Smith <fluendo.com>
  * Copyright (C) <2006> Wim Taymans <wim@fluendo.com>
  * Copyright (C) <2006> Tim-Philipp Müller <tim centricular net>
+ * Copyright (C) <2007> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,12 +27,16 @@
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
+
 #include "rmdemux.h"
 #include "rdtdepay.h"
+#include "rdtmanager.h"
+#include "rtspreal.h"
 #include "rmutils.h"
+#include "rademux.h"
+
 #include <string.h>
 #include <ctype.h>
-#include <zlib.h>
 
 #define RMDEMUX_GUINT32_GET(a)  GST_READ_UINT32_BE(a)
 #define RMDEMUX_GUINT16_GET(a)  GST_READ_UINT16_BE(a)
@@ -39,6 +44,8 @@
 #define HEADER_SIZE 10
 #define DATA_SIZE 8
 
+#define MAX_FRAGS 256
+
 typedef struct _GstRMDemuxIndex GstRMDemuxIndex;
 
 struct _GstRMDemuxStream
@@ -75,6 +82,20 @@ struct _GstRMDemuxStream
   gboolean needs_descrambling;
   guint subpackets_needed;      /* subpackets needed for descrambling    */
   GPtrArray *subpackets;        /* array containing subpacket GstBuffers */
+
+  /* Variables needed for fixing timestamps. */
+  GstClockTime next_ts, last_ts;
+  guint16 next_seq, last_seq;
+
+  gint frag_seqnum;
+  gint frag_subseq;
+  guint frag_length;
+  guint frag_current;
+  guint frag_count;
+  guint frag_offset[MAX_FRAGS];
+  GstAdapter *adapter;
+
+  GstTagList *pending_tags;
 };
 
 struct _GstRMDemuxIndex
@@ -148,7 +169,7 @@ static void gst_rmdemux_parse_data (GstRMDemux * rmdemux, const guint8 * data,
 static void gst_rmdemux_parse_cont (GstRMDemux * rmdemux, const guint8 * data,
     int length);
 static GstFlowReturn gst_rmdemux_parse_packet (GstRMDemux * rmdemux,
-    const guint8 * data, guint16 version, guint16 length);
+    GstBuffer * in, guint16 version);
 static void gst_rmdemux_parse_indx_data (GstRMDemux * rmdemux,
     const guint8 * data, int length);
 static void gst_rmdemux_stream_clear_cached_subpackets (GstRMDemux * rmdemux,
@@ -242,29 +263,31 @@ gst_rmdemux_init (GstRMDemux * rmdemux)
   gst_element_add_pad (GST_ELEMENT (rmdemux), rmdemux->sinkpad);
 
   rmdemux->adapter = gst_adapter_new ();
+  rmdemux->first_ts = GST_CLOCK_TIME_NONE;
+  rmdemux->need_newsegment = TRUE;
 }
 
 static gboolean
 gst_rmdemux_sink_event (GstPad * pad, GstEvent * event)
 {
-  gboolean ret = TRUE;
+  GstRMDemux *rmdemux;
+  gboolean ret;
 
-#ifndef GST_DISABLE_GST_DEBUG
-  GstRMDemux *rmdemux = GST_RMDEMUX (GST_PAD_PARENT (pad));
-#endif
+  rmdemux = GST_RMDEMUX (gst_pad_get_parent (pad));
+
+  GST_LOG_OBJECT (pad, "%s event", GST_EVENT_TYPE_NAME (event));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_NEWSEGMENT:
-      GST_LOG_OBJECT (rmdemux, "Event on sink: NEWSEGMENT");
       gst_event_unref (event);
+      ret = TRUE;
       break;
     default:
-      GST_LOG_OBJECT (rmdemux, "Event on sink: type=%d",
-          GST_EVENT_TYPE (event));
       ret = gst_pad_event_default (pad, event);
       break;
   }
 
+  gst_object_unref (rmdemux);
   return ret;
 }
 
@@ -374,16 +397,15 @@ gst_rmdemux_validate_offset (GstRMDemux * rmdemux)
 static gboolean
 find_seek_offset_bytes (GstRMDemux * rmdemux, guint target)
 {
-  int i, n;
+  int i;
+  GSList *cur;
   gboolean ret = FALSE;
 
   if (target < 0)
     return FALSE;
 
-  for (n = 0; n < rmdemux->n_streams; n++) {
-    GstRMDemuxStream *stream;
-
-    stream = rmdemux->streams[n];
+  for (cur = rmdemux->streams; cur; cur = cur->next) {
+    GstRMDemuxStream *stream = cur->data;
 
     /* Search backwards through this stream's index until we find the first
      * timestamp before our target time */
@@ -404,14 +426,14 @@ find_seek_offset_bytes (GstRMDemux * rmdemux, guint target)
 static gboolean
 find_seek_offset_time (GstRMDemux * rmdemux, GstClockTime time)
 {
-  int i, n;
+  int i, n_stream;
   gboolean ret = FALSE;
+  GSList *cur;
   GstClockTime earliest = GST_CLOCK_TIME_NONE;
 
-  for (n = 0; n < rmdemux->n_streams; n++) {
-    GstRMDemuxStream *stream;
-
-    stream = rmdemux->streams[n];
+  n_stream = 0;
+  for (cur = rmdemux->streams; cur; cur = cur->next, n_stream++) {
+    GstRMDemuxStream *stream = cur->data;
 
     /* Search backwards through this stream's index until we find the first
      * timestamp before our target time */
@@ -431,7 +453,7 @@ find_seek_offset_time (GstRMDemux * rmdemux, GstClockTime time)
           GST_DEBUG_OBJECT (rmdemux,
               "We're looking for %" GST_TIME_FORMAT
               " and we found that stream %d has the latest index at %"
-              GST_TIME_FORMAT, GST_TIME_ARGS (rmdemux->segment.start), n,
+              GST_TIME_FORMAT, GST_TIME_ARGS (rmdemux->segment.start), n_stream,
               GST_TIME_ARGS (earliest));
         }
 
@@ -561,23 +583,14 @@ gst_rmdemux_perform_seek (GstRMDemux * rmdemux, GstEvent * event)
 
   /* now we have a new position, prepare for streaming again */
   {
-    GstEvent *event;
-
     /* Reset the demuxer state */
     rmdemux->state = RMDEMUX_STATE_DATA_PACKET;
 
     if (flush)
       gst_rmdemux_send_event (rmdemux, gst_event_new_flush_stop ());
 
-    /* create the discont event we are going to send out */
-    event = gst_event_new_new_segment (FALSE, rmdemux->segment.rate,
-        rmdemux->segment.format, rmdemux->segment.start,
-        rmdemux->segment.stop, rmdemux->segment.time);
-
-    GST_DEBUG_OBJECT (rmdemux,
-        "sending NEWSEGMENT event to all src pads with segment.start= %"
-        GST_TIME_FORMAT, GST_TIME_ARGS (rmdemux->segment.start));
-    gst_rmdemux_send_event (rmdemux, event);
+    /* must send newsegment event from streaming thread, so just set flag */
+    rmdemux->need_newsegment = TRUE;
 
     /* notify start of new segment */
     if (rmdemux->segment.flags & GST_SEEK_FLAG_SEGMENT) {
@@ -585,6 +598,7 @@ gst_rmdemux_perform_seek (GstRMDemux * rmdemux, GstEvent * event)
           gst_message_new_segment_start (GST_OBJECT_CAST (rmdemux),
               GST_FORMAT_TIME, rmdemux->segment.last_stop));
     }
+
     /* restart our task since it might have been stopped when we did the 
      * flush. */
     gst_pad_start_task (rmdemux->sinkpad, (GstTaskFunction) gst_rmdemux_loop,
@@ -608,29 +622,52 @@ error:
 static gboolean
 gst_rmdemux_src_query (GstPad * pad, GstQuery * query)
 {
-  gboolean res = TRUE;
+  gboolean res = FALSE;
   GstRMDemux *rmdemux;
 
-  rmdemux = GST_RMDEMUX (GST_PAD_PARENT (pad));
+  rmdemux = GST_RMDEMUX (gst_pad_get_parent (pad));
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_POSITION:
-      GST_DEBUG_OBJECT (rmdemux, "src_query position");
-      gst_query_set_position (query, GST_FORMAT_TIME, -1);
       GST_DEBUG_OBJECT (rmdemux, "Position query: no idea from demuxer!");
-      res = FALSE;
       break;
-    case GST_QUERY_DURATION:
-      GST_DEBUG_OBJECT (rmdemux, "src_query duration");
-      gst_query_set_duration (query, GST_FORMAT_TIME, rmdemux->duration);
-      GST_DEBUG_OBJECT (rmdemux, "duration set to %" G_GINT64_FORMAT,
-          rmdemux->duration);
+    case GST_QUERY_DURATION:{
+      GstFormat fmt;
+
+      gst_query_parse_duration (query, &fmt, NULL);
+      if (fmt == GST_FORMAT_TIME) {
+        GST_OBJECT_LOCK (rmdemux);
+        if (G_LIKELY (rmdemux->running)) {
+          gst_query_set_duration (query, GST_FORMAT_TIME, rmdemux->duration);
+          GST_DEBUG_OBJECT (rmdemux, "duration set to %" GST_TIME_FORMAT,
+              GST_TIME_ARGS (rmdemux->duration));
+          res = TRUE;
+        }
+        GST_OBJECT_UNLOCK (rmdemux);
+      }
       break;
+    }
+    case GST_QUERY_SEEKING:{
+      GstFormat fmt;
+
+      gst_query_parse_seeking (query, &fmt, NULL, NULL, NULL);
+      if (fmt == GST_FORMAT_TIME) {
+        GST_OBJECT_LOCK (rmdemux);
+        if (G_LIKELY (rmdemux->running)) {
+          gst_query_set_seeking (query, GST_FORMAT_TIME, rmdemux->seekable,
+              0, rmdemux->duration);
+          res = TRUE;
+        }
+        GST_OBJECT_UNLOCK (rmdemux);
+      }
+      break;
+    }
     default:
       res = gst_pad_query_default (pad, query);
       break;
   }
 
+  gst_object_unref (rmdemux);
   return res;
 }
 
@@ -640,6 +677,7 @@ gst_rmdemux_src_query_types (GstPad * pad)
   static const GstQueryType query_types[] = {
     GST_QUERY_POSITION,
     GST_QUERY_DURATION,
+    GST_QUERY_SEEKING,
     0
   };
 
@@ -649,21 +687,27 @@ gst_rmdemux_src_query_types (GstPad * pad)
 static void
 gst_rmdemux_reset (GstRMDemux * rmdemux)
 {
-  guint n;
+  GSList *cur;
 
   GST_OBJECT_LOCK (rmdemux);
   rmdemux->running = FALSE;
   GST_OBJECT_UNLOCK (rmdemux);
 
-  for (n = 0; n < rmdemux->n_streams; ++n) {
-    gst_rmdemux_stream_clear_cached_subpackets (rmdemux, rmdemux->streams[n]);
-    gst_element_remove_pad (GST_ELEMENT (rmdemux), rmdemux->streams[n]->pad);
-    if (rmdemux->streams[n]->subpackets)
-      g_ptr_array_free (rmdemux->streams[n]->subpackets, TRUE);
-    g_free (rmdemux->streams[n]);
-    rmdemux->streams[n] = NULL;
+  for (cur = rmdemux->streams; cur; cur = cur->next) {
+    GstRMDemuxStream *stream = cur->data;
+
+    g_object_unref (stream->adapter);
+    gst_rmdemux_stream_clear_cached_subpackets (rmdemux, stream);
+    gst_element_remove_pad (GST_ELEMENT (rmdemux), stream->pad);
+    if (stream->pending_tags)
+      gst_tag_list_free (stream->pending_tags);
+    if (stream->subpackets)
+      g_ptr_array_free (stream->subpackets, TRUE);
+    g_free (stream->index);
+    g_free (stream);
   }
-  rmdemux->n_streams = 0;
+  g_slist_free (rmdemux->streams);
+  rmdemux->streams = NULL;
   rmdemux->n_audio_streams = 0;
   rmdemux->n_video_streams = 0;
 
@@ -672,6 +716,8 @@ gst_rmdemux_reset (GstRMDemux * rmdemux)
   rmdemux->have_pads = FALSE;
 
   gst_segment_init (&rmdemux->segment, GST_FORMAT_UNDEFINED);
+  rmdemux->first_ts = GST_CLOCK_TIME_NONE;
+  rmdemux->need_newsegment = TRUE;
 }
 
 static GstStateChangeReturn
@@ -921,16 +967,19 @@ gst_rmdemux_chain (GstPad * pad, GstBuffer * buffer)
   GstFlowReturn ret = GST_FLOW_OK;
   const guint8 *data;
   guint16 version;
+  guint avail;
 
   GstRMDemux *rmdemux = GST_RMDEMUX (GST_PAD_PARENT (pad));
 
+  gst_adapter_push (rmdemux->adapter, buffer);
+
   GST_LOG_OBJECT (rmdemux, "Chaining buffer of size %d",
       GST_BUFFER_SIZE (buffer));
 
-  gst_adapter_push (rmdemux->adapter, buffer);
-
   while (TRUE) {
-    GST_LOG_OBJECT (rmdemux, "looping in chain");
+    avail = gst_adapter_available (rmdemux->adapter);
+
+    GST_LOG_OBJECT (rmdemux, "looping in chain, avail %u", avail);
     switch (rmdemux->state) {
       case RMDEMUX_STATE_HEADER:
       {
@@ -1056,18 +1105,14 @@ gst_rmdemux_chain (GstPad * pad, GstBuffer * buffer)
       {
         /* If we haven't already done so then signal there are no more pads */
         if (!rmdemux->have_pads) {
+          GST_LOG_OBJECT (rmdemux, "no more pads");
           gst_element_no_more_pads (GST_ELEMENT (rmdemux));
           rmdemux->have_pads = TRUE;
-
-          GST_LOG_OBJECT (rmdemux, "no more pads.");
-          gst_rmdemux_send_event (rmdemux,
-              gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_TIME,
-                  (gint64) 0, (gint64) - 1, 0));
         }
 
         /* The actual header is only 8 bytes */
         rmdemux->size = DATA_SIZE;
-        GST_DEBUG_OBJECT (rmdemux, "data available %d",
+        GST_LOG_OBJECT (rmdemux, "data available %d",
             gst_adapter_available (rmdemux->adapter));
         if (gst_adapter_available (rmdemux->adapter) < rmdemux->size)
           goto unlock;
@@ -1129,20 +1174,30 @@ gst_rmdemux_chain (GstPad * pad, GstBuffer * buffer)
           data = gst_adapter_peek (rmdemux->adapter, 4);
 
           length = RMDEMUX_GUINT16_GET (data + 2);
+          GST_LOG_OBJECT (rmdemux, "Got length %d", length);
+
           if (length < 4) {
+            GST_LOG_OBJECT (rmdemux, "length too small, dropping");
             /* Invalid, just drop it */
             gst_adapter_flush (rmdemux->adapter, 4);
           } else {
-            if (gst_adapter_available (rmdemux->adapter) < length)
+            GstBuffer *buffer;
+
+            avail = gst_adapter_available (rmdemux->adapter);
+            if (avail < length)
               goto unlock;
-            data = gst_adapter_peek (rmdemux->adapter, length);
 
-            ret =
-                gst_rmdemux_parse_packet (rmdemux, data + 4, version,
-                length - 4);
+            GST_LOG_OBJECT (rmdemux, "we have %u available and we needed %d",
+                avail, length);
+
+            /* flush version and length */
+            gst_adapter_flush (rmdemux->adapter, 4);
+            length -= 4;
+
+            buffer = gst_adapter_take_buffer (rmdemux->adapter, length);
+
+            ret = gst_rmdemux_parse_packet (rmdemux, buffer, version);
             rmdemux->chunk_index++;
-
-            gst_adapter_flush (rmdemux->adapter, length);
           }
 
           if (rmdemux->chunk_index == rmdemux->n_chunks || length == 0)
@@ -1176,12 +1231,11 @@ unlock:
 static GstRMDemuxStream *
 gst_rmdemux_get_stream_by_id (GstRMDemux * rmdemux, int id)
 {
-  int i;
+  GSList *cur;
 
-  for (i = 0; i < rmdemux->n_streams; i++) {
-    GstRMDemuxStream *stream;
+  for (cur = rmdemux->streams; cur; cur = cur->next) {
+    GstRMDemuxStream *stream = cur->data;
 
-    stream = rmdemux->streams[i];
     if (stream->id == id) {
       return stream;
     }
@@ -1193,10 +1247,10 @@ gst_rmdemux_get_stream_by_id (GstRMDemux * rmdemux, int id)
 static void
 gst_rmdemux_send_event (GstRMDemux * rmdemux, GstEvent * event)
 {
-  int i;
+  GSList *cur;
 
-  for (i = 0; i < rmdemux->n_streams; i++) {
-    GstRMDemuxStream *stream = rmdemux->streams[i];
+  for (cur = rmdemux->streams; cur; cur = cur->next) {
+    GstRMDemuxStream *stream = cur->data;
 
     GST_DEBUG_OBJECT (rmdemux, "Pushing %s event on pad %s",
         GST_EVENT_TYPE_NAME (event), GST_PAD_NAME (stream->pad));
@@ -1376,9 +1430,12 @@ gst_rmdemux_add_stream (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
   }
 
   GST_PAD_ELEMENT_PRIVATE (stream->pad) = stream;
-  rmdemux->streams[rmdemux->n_streams] = stream;
-  rmdemux->n_streams++;
-  GST_LOG_OBJECT (rmdemux, "n_streams is now %d", rmdemux->n_streams);
+  rmdemux->streams = g_slist_append (rmdemux->streams, stream);
+  GST_LOG_OBJECT (rmdemux, "n_streams is now %d",
+      g_slist_length (rmdemux->streams));
+
+  GST_LOG ("stream->pad = %p, stream_caps = %" GST_PTR_FORMAT, stream->pad,
+      stream_caps);
 
   if (stream->pad && stream_caps) {
 
@@ -1414,16 +1471,11 @@ gst_rmdemux_add_stream (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
     gst_pad_set_active (stream->pad, TRUE);
     gst_element_add_pad (GST_ELEMENT_CAST (rmdemux), stream->pad);
 
-    gst_pad_push_event (stream->pad,
-        gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_TIME, (gint64) 0,
-            (gint64) - 1, 0));
-
-    if (codec_name && codec_tag) {
-      GstTagList *tags = NULL;
-
-      tags = gst_tag_list_new ();
-      gst_tag_list_add (tags, GST_TAG_MERGE_KEEP, codec_tag, codec_name, NULL);
-      gst_element_found_tags_for_pad (GST_ELEMENT (rmdemux), stream->pad, tags);
+    /* save for later, we must send the tags after the newsegment event */
+    if (codec_name != NULL && codec_tag != NULL) {
+      stream->pending_tags = gst_tag_list_new ();
+      gst_tag_list_add (stream->pending_tags, GST_TAG_MERGE_KEEP,
+          codec_tag, codec_name, NULL);
     }
   }
 
@@ -1486,14 +1538,13 @@ gst_rmdemux_parse_mdpr (GstRMDemux * rmdemux, const guint8 * data, int length)
   int stream_type;
   int offset;
 
-  /* re_hexdump_bytes ((guint8 *) data, length, 0); */
-
   stream = g_new0 (GstRMDemuxStream, 1);
 
   stream->id = RMDEMUX_GUINT16_GET (data);
   stream->index = NULL;
   stream->seek_offset = 0;
   stream->last_flow = GST_FLOW_OK;
+  stream->adapter = gst_adapter_new ();
   GST_LOG_OBJECT (rmdemux, "stream_number=%d", stream->id);
 
   offset = 30;
@@ -1556,8 +1607,8 @@ gst_rmdemux_parse_mdpr (GstRMDemux * rmdemux, const guint8 * data, int length)
       stream->rate = RMDEMUX_GUINT16_GET (data + offset + 16);
       stream->subformat = RMDEMUX_GUINT32_GET (data + offset + 26);
       stream->format = RMDEMUX_GUINT32_GET (data + offset + 30);
-      stream->extra_data_size = length - (offset + 34);
-      stream->extra_data = (guint8 *) data + offset + 34;
+      stream->extra_data_size = length - (offset + 26);
+      stream->extra_data = (guint8 *) data + offset + 26;
       /* Natural way to represent framerates here requires unsigned 32 bit
        * numerator, which we don't have. For the nasty case, approximate...
        */
@@ -1726,6 +1777,13 @@ gst_rmdemux_parse_indx_data (GstRMDemux * rmdemux, const guint8 * data,
   if (rmdemux->index_stream == NULL)
     return;
 
+  /* don't parse the index a second time when operating pull-based and
+   * reaching the end of the file */
+  if (rmdemux->index_stream->index_length > 0) {
+    GST_DEBUG_OBJECT (rmdemux, "Already have an index for this stream");
+    return;
+  }
+
   index = g_malloc (sizeof (GstRMDemuxIndex) * n);
   rmdemux->index_stream->index = index;
   rmdemux->index_stream->index_length = n;
@@ -1735,7 +1793,8 @@ gst_rmdemux_parse_indx_data (GstRMDemux * rmdemux, const guint8 * data,
     index[i].offset = RMDEMUX_GUINT32_GET (data + 6);
 
     GST_DEBUG_OBJECT (rmdemux, "Index found for timestamp=%f (at offset=%x)",
-        (float) index[i].timestamp / GST_SECOND, index[i].offset);
+        gst_guint64_to_gdouble (index[i].timestamp) / GST_SECOND,
+        index[i].offset);
     data += 14;
   }
 }
@@ -1765,7 +1824,7 @@ static GstFlowReturn
 gst_rmdemux_combine_flows (GstRMDemux * rmdemux, GstRMDemuxStream * stream,
     GstFlowReturn ret)
 {
-  gint i;
+  GSList *cur;
 
   /* store the value */
   stream->last_flow = ret;
@@ -1779,8 +1838,8 @@ gst_rmdemux_combine_flows (GstRMDemux * rmdemux, GstRMDemuxStream * stream,
   if (ret != GST_FLOW_NOT_LINKED)
     goto done;
 
-  for (i = 0; i < rmdemux->n_streams; ++i) {
-    GstRMDemuxStream *ostream = rmdemux->streams[i];
+  for (cur = rmdemux->streams; cur; cur = cur->next) {
+    GstRMDemuxStream *ostream = cur->data;
 
     ret = ostream->last_flow;
     /* some other return value (must be SUCCESS but we can return
@@ -1847,7 +1906,18 @@ gst_rmdemux_descramble_cook_audio (GstRMDemux * rmdemux,
     }
   }
 
-  ret = gst_pad_push (stream->pad, outbuf);
+  /* some decoders, such as realaudiodec, need to be fed in packet units */
+  for (p = 0; p < height; ++p) {
+    GstBuffer *subbuf;
+
+    subbuf = gst_buffer_create_sub (outbuf, p * packet_size, packet_size);
+    gst_buffer_set_caps (subbuf, GST_PAD_CAPS (stream->pad));
+    ret = gst_pad_push (stream->pad, subbuf);
+    if (ret != GST_FLOW_OK)
+      break;
+  }
+
+  gst_buffer_unref (outbuf);
 
 done:
 
@@ -1861,20 +1931,12 @@ gst_rmdemux_descramble_dnet_audio (GstRMDemux * rmdemux,
     GstRMDemuxStream * stream)
 {
   GstBuffer *buf;
-  guint16 *data, *end;
 
   buf = g_ptr_array_index (stream->subpackets, 0);
   g_ptr_array_index (stream->subpackets, 0) = NULL;
   g_ptr_array_set_size (stream->subpackets, 0);
 
-  /* descramble is a bit of a misnomer, it's just byte-order swapped AC3 .. */
-  data = (guint16 *) GST_BUFFER_DATA (buf);
-  end = (guint16 *) (GST_BUFFER_DATA (buf) + GST_BUFFER_SIZE (buf));
-  while (data < end) {
-    *data = GUINT16_SWAP_LE_BE (*data);
-    ++data;
-  }
-
+  buf = gst_rm_utils_descramble_dnet_buffer (buf);
   return gst_pad_push (stream->pad, buf);
 }
 
@@ -1915,66 +1977,348 @@ gst_rmdemux_handle_scrambled_packet (GstRMDemux * rmdemux,
   return ret;
 }
 
-static GstFlowReturn
-gst_rmdemux_parse_packet (GstRMDemux * rmdemux, const guint8 * data,
-    guint16 version, guint16 length)
+static GstClockTime
+gst_rmdemux_fix_timestamp (GstRMDemux * rmdemux, GstRMDemuxStream * stream,
+    guint8 * data, GstClockTime timestamp)
 {
-  guint16 id;
-  GstRMDemuxStream *stream;
-  GstBuffer *buffer;
-  guint16 packet_size;
-  GstFlowReturn cret, ret;
-  GstClockTime timestamp;
-  gboolean key = FALSE;
+  guint8 frame_type;
+  guint16 seq;
+  GstClockTime ts = timestamp;
 
-  id = RMDEMUX_GUINT16_GET (data);
-  /* timestamp in Msec */
-  timestamp = RMDEMUX_GUINT32_GET (data + 2) * GST_MSECOND;
+  if (timestamp == GST_CLOCK_TIME_NONE)
+    goto done;
 
-  gst_segment_set_last_stop (&rmdemux->segment, GST_FORMAT_TIME, timestamp);
+  /* only adjust when we have a stream with B frames */
+  if (stream->format < 0x20200002)
+    goto done;
 
-  GST_LOG_OBJECT (rmdemux, "Parsing a packet for stream=%d, timestamp=%"
-      GST_TIME_FORMAT ", version=%d", id, GST_TIME_ARGS (timestamp), version);
-
-  /* skip stream_id and timestamp */
-  data += 2 + 4;
-  packet_size = length - (2 + 4);
-
-  /* skip other stuff */
-  if (version == 0) {
-    /* uint8 packet_group */
-    /* uint8 flags        */
-    key = ((GST_READ_UINT8 (data + 1) & 0x02) != 0);
-    data += 2;
-    packet_size -= 2;
-  } else {
-    /* uint16 asm_rule */
-    /* uint8 asm_flags */
-    data += 3;
-    packet_size -= 3;
+  /* Fix timestamp. */
+  switch (stream->fourcc) {
+    case GST_RM_VDO_RV10:
+      goto done;
+    case GST_RM_VDO_RV20:
+    {
+      /*
+       * Bit  1- 2: frame type
+       * Bit  3- 9: ?
+       * Bit 10-22: sequence number
+       * Bit 23-32: ?
+       */
+      frame_type = (data[0] >> 6) & 0x03;
+      seq = ((data[1] & 0x7f) << 6) + ((data[2] & 0xfc) >> 2);
+      break;
+    }
+    case GST_RM_VDO_RV30:
+    {
+      /*
+       * Bit  1- 2: ?
+       * Bit     3: skip packet if 1
+       * Bit  4- 5: frame type
+       * Bit  6-12: ?
+       * Bit 13-25: sequence number
+       * Bit 26-32: ?
+       */
+      frame_type = (data[0] >> 3) & 0x03;
+      seq = ((data[1] & 0x0f) << 9) + (data[2] << 1) + ((data[3] & 0x80) >> 7);
+      break;
+    }
+    case GST_RM_VDO_RV40:
+    {
+      /*
+       * Bit     1: skip packet if 1
+       * Bit  2- 3: frame type
+       * Bit  4-13: ?
+       * Bit 14-26: sequence number
+       * Bit 27-32: ?
+       */
+      frame_type = (data[0] >> 5) & 0x03;
+      seq = ((data[1] & 0x07) << 10) + (data[2] << 2) + ((data[3] & 0xc0) >> 6);
+      break;
+    }
+    default:
+      goto unknown_version;
   }
 
-  stream = gst_rmdemux_get_stream_by_id (rmdemux, id);
-  if (!stream || !stream->pad)
-    goto unknown_stream;
+  switch (frame_type) {
+    case 0:
+    case 1:
+    {
+      GST_LOG_OBJECT (rmdemux, "I frame %d", frame_type);
+      /* I frame */
+      timestamp = stream->next_ts;
+      stream->last_ts = stream->next_ts;
+      stream->next_ts = ts;
+      stream->last_seq = stream->next_seq;
+      stream->next_seq = seq;
+      break;
+    }
+    case 2:
+    {
+      GST_LOG_OBJECT (rmdemux, "P frame");
+      /* P frame */
+      timestamp = stream->last_ts = stream->next_ts;
+      if (seq < stream->next_seq)
+        stream->next_ts += (seq + 0x2000 - stream->next_seq) * GST_MSECOND;
+      else
+        stream->next_ts += (seq - stream->next_seq) * GST_MSECOND;
+      stream->last_seq = stream->next_seq;
+      stream->next_seq = seq;
+      break;
+    }
+    case 3:
+    {
+      GST_LOG_OBJECT (rmdemux, "B frame");
+      /* B frame */
+      if (seq < stream->last_seq) {
+        timestamp =
+            (seq + 0x2000 - stream->last_seq) * GST_MSECOND + stream->last_ts;
+      } else {
+        timestamp = (seq - stream->last_seq) * GST_MSECOND + stream->last_ts;
+      }
+      break;
+    }
+    default:
+      goto unknown_frame_type;
+  }
+
+done:
+  GST_LOG_OBJECT (rmdemux,
+      "timestamp %" GST_TIME_FORMAT " -> %" GST_TIME_FORMAT, GST_TIME_ARGS (ts),
+      GST_TIME_ARGS (timestamp));
+
+  return timestamp;
+
+  /* Errors */
+unknown_version:
+  {
+    GST_ELEMENT_ERROR (rmdemux, STREAM, DECODE,
+        ("Unknown version: %i.", stream->version), (NULL));
+    return GST_FLOW_ERROR;
+  }
+
+unknown_frame_type:
+  {
+    GST_ELEMENT_ERROR (rmdemux, STREAM, DECODE, ("Unknown frame type %d.",
+            frame_type), (NULL));
+    return GST_FLOW_ERROR;
+  }
+}
+
+#define PARSE_NUMBER(data, size, number, label) \
+G_STMT_START {                                  \
+  if (size < 2)                                 \
+    goto label;                                 \
+  number = GST_READ_UINT16_BE (data);           \
+  if (!(number & 0xc000)) {                     \
+    if (size < 4)                               \
+      goto label;                               \
+    number = GST_READ_UINT32_BE (data);         \
+    data += 4;                                  \
+    size -= 4;                                  \
+  } else {                                      \
+    number &= 0x3fff;                           \
+    data += 2;                                  \
+    size -= 2;                                  \
+  }                                             \
+} G_STMT_END
+
+static GstFlowReturn
+gst_rmdemux_parse_video_packet (GstRMDemux * rmdemux, GstRMDemuxStream * stream,
+    GstBuffer * in, guint offset, guint16 version,
+    GstClockTime timestamp, gboolean key)
+{
+  GstFlowReturn ret;
+  const guint8 *data, *base;
+  guint size;
+
+  base = GST_BUFFER_DATA (in);
+  data = base + offset;
+  size = GST_BUFFER_SIZE (in) - offset;
+
+  while (size > 2) {
+    guint8 pkg_header;
+    guint pkg_offset;
+    guint pkg_length;
+    guint pkg_subseq = 0, pkg_seqnum = G_MAXUINT;
+    guint fragment_size;
+    GstBuffer *fragment;
+
+    pkg_header = *data++;
+    size--;
+
+    /* packet header
+     * bit 7: 1=last block in block chain
+     * bit 6: 1=short header (only one block?)
+     */
+    if ((pkg_header & 0xc0) == 0x40) {
+      /* skip unknown byte */
+      data++;
+      size--;
+      pkg_offset = 0;
+      pkg_length = size;
+    } else {
+      if ((pkg_header & 0x40) == 0) {
+        pkg_subseq = (*data++) & 0x7f;
+        size--;
+      } else {
+        pkg_subseq = 0;
+      }
+
+      /* length */
+      PARSE_NUMBER (data, size, pkg_length, not_enough_data);
+
+      /* offset */
+      PARSE_NUMBER (data, size, pkg_offset, not_enough_data);
+
+      /* seqnum */
+      if (size < 1)
+        goto not_enough_data;
+
+      pkg_seqnum = *data++;
+      size--;
+    }
 
-  if ((rmdemux->offset + packet_size) <= stream->seek_offset) {
     GST_DEBUG_OBJECT (rmdemux,
-        "Stream %d is skipping: seek_offset=%d, offset=%d, packet_size=%u",
-        stream->id, stream->seek_offset, rmdemux->offset, packet_size);
-    cret = GST_FLOW_OK;
-    goto beach;
+        "seq %d, subseq %d, offset %d, length %d, size %d, header %02x",
+        pkg_seqnum, pkg_subseq, pkg_offset, pkg_length, size, pkg_header);
+
+    /* calc size of fragment */
+    if ((pkg_header & 0xc0) == 0x80) {
+      fragment_size = pkg_offset;
+    } else {
+      if ((pkg_header & 0xc0) == 0)
+        fragment_size = size;
+      else
+        fragment_size = pkg_length;
+    }
+    GST_DEBUG_OBJECT (rmdemux, "fragment size %d", fragment_size);
+
+    /* get the fragment */
+    fragment = gst_buffer_create_sub (in, data - base, fragment_size);
+
+    if (pkg_subseq == 1) {
+      GST_DEBUG_OBJECT (rmdemux, "start new fragment");
+      gst_adapter_clear (stream->adapter);
+      stream->frag_current = 0;
+      stream->frag_count = 0;
+      stream->frag_length = pkg_length;
+    } else if (pkg_subseq == 0) {
+      GST_DEBUG_OBJECT (rmdemux, "non fragmented packet");
+      stream->frag_current = 0;
+      stream->frag_count = 0;
+      stream->frag_length = fragment_size;
+    }
+
+    /* put fragment in adapter */
+    gst_adapter_push (stream->adapter, fragment);
+    stream->frag_offset[stream->frag_count] = stream->frag_current;
+    stream->frag_current += fragment_size;
+    stream->frag_count++;
+
+    if (stream->frag_count > MAX_FRAGS)
+      goto too_many_fragments;
+
+    GST_DEBUG_OBJECT (rmdemux, "stored fragment in adapter %d/%d",
+        stream->frag_current, stream->frag_length);
+
+    /* flush fragment when complete */
+    if (stream->frag_current >= stream->frag_length) {
+      GstBuffer *out;
+      guint8 *outdata;
+      guint header_size;
+      gint i;
+
+      /* calculate header size, which is:
+       * 1 byte for the number of fragments - 1
+       * for each fragment:
+       *   4 bytes 0x00000001 little endian
+       *   4 bytes fragment offset
+       *
+       * This is also the matroska header for realvideo, the decoder needs the
+       * fragment offsets, both in ffmpeg and real .so, so we just give it that
+       * in front of the data.
+       */
+      header_size = 1 + (8 * (stream->frag_count));
+
+      GST_DEBUG_OBJECT (rmdemux,
+          "fragmented completed. count %d, header_size %u", stream->frag_count,
+          header_size);
+
+      out = gst_buffer_new_and_alloc (header_size + stream->frag_length);
+      outdata = GST_BUFFER_DATA (out);
+
+      /* create header */
+      *outdata++ = stream->frag_count - 1;
+      for (i = 0; i < stream->frag_count; i++) {
+        GST_WRITE_UINT32_LE (outdata, 0x00000001);
+        outdata += 4;
+        GST_WRITE_UINT32_LE (outdata, stream->frag_offset[i]);
+        outdata += 4;
+      }
+
+      /* copy packet data after the header now */
+      gst_adapter_copy (stream->adapter, outdata, 0, stream->frag_length);
+      gst_adapter_flush (stream->adapter, stream->frag_length);
+
+      gst_buffer_set_caps (out, GST_PAD_CAPS (stream->pad));
+      GST_BUFFER_TIMESTAMP (out) =
+          gst_rmdemux_fix_timestamp (rmdemux, stream, outdata, timestamp);
+
+      ret = gst_pad_push (stream->pad, out);
+
+      timestamp = GST_CLOCK_TIME_NONE;
+    }
+    data += fragment_size;
+    size -= fragment_size;
   }
+  GST_DEBUG_OBJECT (rmdemux, "%d bytes left", size);
+
+  gst_buffer_unref (in);
+
+  ret = GST_FLOW_OK;
+
+  return ret;
+
+  /* ERRORS */
+not_enough_data:
+  {
+    GST_ELEMENT_WARNING (rmdemux, STREAM, DECODE, ("Skipping bad packet."),
+        (NULL));
+    gst_buffer_unref (in);
+    return GST_FLOW_OK;
+  }
+too_many_fragments:
+  {
+    GST_ELEMENT_ERROR (rmdemux, STREAM, DECODE,
+        ("Got more fragments (%u) than can be handled (%u)",
+            stream->frag_count, MAX_FRAGS), (NULL));
+    gst_buffer_unref (in);
+    return GST_FLOW_ERROR;
+  }
+}
+
+static GstFlowReturn
+gst_rmdemux_parse_audio_packet (GstRMDemux * rmdemux, GstRMDemuxStream * stream,
+    GstBuffer * in, guint offset, guint16 version,
+    GstClockTime timestamp, gboolean key)
+{
+  GstFlowReturn ret, cret;
+  GstBuffer *buffer;
+  const guint8 *data;
+  guint size;
+
+  data = GST_BUFFER_DATA (in) + offset;
+  size = GST_BUFFER_SIZE (in) - offset;
 
   ret = gst_pad_alloc_buffer_and_set_caps (stream->pad,
-      GST_BUFFER_OFFSET_NONE, packet_size, GST_PAD_CAPS (stream->pad), &buffer);
+      GST_BUFFER_OFFSET_NONE, size, GST_PAD_CAPS (stream->pad), &buffer);
 
   cret = gst_rmdemux_combine_flows (rmdemux, stream, ret);
   if (ret != GST_FLOW_OK)
     goto alloc_failed;
 
-  memcpy (GST_BUFFER_DATA (buffer), (guint8 *) data, packet_size);
-  GST_BUFFER_TIMESTAMP (buffer) = timestamp;
+  memcpy (GST_BUFFER_DATA (buffer), (guint8 *) data, size);
+  GST_BUFFER_TIMESTAMP (buffer) = timestamp - rmdemux->first_ts;
 
   if (stream->needs_descrambling) {
     ret = gst_rmdemux_handle_scrambled_packet (rmdemux, stream, buffer, key);
@@ -1984,6 +2328,114 @@ gst_rmdemux_parse_packet (GstRMDemux * rmdemux, const guint8 * data,
 
     ret = gst_pad_push (stream->pad, buffer);
   }
+  return ret;
+
+  /* ERRORS */
+alloc_failed:
+  {
+    GST_DEBUG_OBJECT (rmdemux, "pad alloc returned %d", ret);
+    return cret;
+  }
+}
+
+static GstFlowReturn
+gst_rmdemux_parse_packet (GstRMDemux * rmdemux, GstBuffer * in, guint16 version)
+{
+  guint16 id;
+  GstRMDemuxStream *stream;
+  guint size;
+  GstFlowReturn cret, ret;
+  GstClockTime timestamp;
+  gboolean key;
+  guint8 *data, *base;
+  guint8 flags;
+
+  base = data = GST_BUFFER_DATA (in);
+  size = GST_BUFFER_SIZE (in);
+
+  /* stream number */
+  id = RMDEMUX_GUINT16_GET (data);
+
+  stream = gst_rmdemux_get_stream_by_id (rmdemux, id);
+  if (!stream || !stream->pad)
+    goto unknown_stream;
+
+  /* timestamp in Msec */
+  timestamp = RMDEMUX_GUINT32_GET (data + 2) * GST_MSECOND;
+
+  gst_segment_set_last_stop (&rmdemux->segment, GST_FORMAT_TIME, timestamp);
+
+  GST_LOG_OBJECT (rmdemux, "Parsing a packet for stream=%d, timestamp=%"
+      GST_TIME_FORMAT ", size %u, version=%d", id, GST_TIME_ARGS (timestamp),
+      size, version);
+
+  if (rmdemux->first_ts == GST_CLOCK_TIME_NONE) {
+    GST_DEBUG_OBJECT (rmdemux, "First timestamp: %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (timestamp));
+    rmdemux->first_ts = timestamp;
+  }
+
+  /* skip stream_id and timestamp */
+  data += (2 + 4);
+  size -= (2 + 4);
+
+  /* skip other stuff */
+  if (version == 0) {
+    /* uint8 packet_group */
+    /* uint8 flags        */
+    flags = GST_READ_UINT8 (data + 1);
+    data += 2;
+    size -= 2;
+  } else {
+    /* uint16 asm_rule */
+    /* uint8 asm_flags */
+    flags = GST_READ_UINT8 (data + 2);
+    data += 3;
+    size -= 3;
+  }
+  key = (flags & 0x02) != 0;
+  GST_DEBUG_OBJECT (rmdemux, "flags %d, Keyframe %d", flags, key);
+
+  if (rmdemux->need_newsegment) {
+    GstEvent *event;
+
+    event = gst_event_new_new_segment (FALSE, rmdemux->segment.rate,
+        rmdemux->segment.format, rmdemux->segment.start,
+        rmdemux->segment.stop, rmdemux->segment.time);
+
+    GST_DEBUG_OBJECT (rmdemux, "sending NEWSEGMENT event, segment.start= %"
+        GST_TIME_FORMAT, GST_TIME_ARGS (rmdemux->segment.start));
+
+    gst_rmdemux_send_event (rmdemux, event);
+    rmdemux->need_newsegment = FALSE;
+  }
+
+  if (stream->pending_tags != NULL) {
+    GST_LOG_OBJECT (stream->pad, "tags %" GST_PTR_FORMAT, stream->pending_tags);
+    gst_element_found_tags_for_pad (GST_ELEMENT_CAST (rmdemux), stream->pad,
+        stream->pending_tags);
+    stream->pending_tags = NULL;
+  }
+
+  if ((rmdemux->offset + size) <= stream->seek_offset) {
+    GST_DEBUG_OBJECT (rmdemux,
+        "Stream %d is skipping: seek_offset=%d, offset=%d, size=%u",
+        stream->id, stream->seek_offset, rmdemux->offset, size);
+    cret = GST_FLOW_OK;
+    goto beach;
+  }
+
+  /* do special headers */
+  if (stream->subtype == GST_RMDEMUX_STREAM_VIDEO) {
+    ret =
+        gst_rmdemux_parse_video_packet (rmdemux, stream, in, data - base,
+        version, timestamp, key);
+  } else if (stream->subtype == GST_RMDEMUX_STREAM_AUDIO) {
+    ret =
+        gst_rmdemux_parse_audio_packet (rmdemux, stream, in, data - base,
+        version, timestamp, key);
+  } else
+    ret = GST_FLOW_OK;
 
   cret = gst_rmdemux_combine_flows (rmdemux, stream, ret);
 
@@ -1997,18 +2449,15 @@ unknown_stream:
         "data packet", id);
     return GST_FLOW_OK;
   }
-alloc_failed:
-  {
-    GST_DEBUG_OBJECT (rmdemux, "pad alloc returned %d", ret);
-    return cret;
-  }
 }
 
-gboolean
+static gboolean
 gst_rmdemux_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rmdemux",
-      GST_RANK_PRIMARY, GST_TYPE_RMDEMUX);
+      GST_RANK_PRIMARY, GST_TYPE_RMDEMUX) &&
+      gst_element_register (plugin, "rademux",
+      GST_RANK_SECONDARY, GST_TYPE_REAL_AUDIO_DEMUX);
 }
 
 
@@ -2021,6 +2470,12 @@ plugin_init (GstPlugin * plugin)
   if (!gst_rdt_depay_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rdt_manager_plugin_init (plugin))
+    return FALSE;
+
+  if (!gst_rtsp_real_plugin_init (plugin))
+    return FALSE;
+
   return TRUE;
 }
 
