@@ -1,4 +1,4 @@
-/* GStreamer
+/* GStreamer RealMedia demuxer
  * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
  *
  * This library is free software; you can redistribute it and/or
@@ -37,8 +37,6 @@ G_BEGIN_DECLS
 #define GST_IS_RMDEMUX_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_RMDEMUX))
 
-#define GST_RMDEMUX_MAX_STREAMS         8
-
 typedef enum
 {
   RMDEMUX_STATE_NULL,
@@ -83,8 +81,7 @@ struct _GstRMDemux {
   /* pads */
   GstPad *sinkpad;
 
-  GstRMDemuxStream *streams[GST_RMDEMUX_MAX_STREAMS];
-  int n_streams;
+  GSList *streams;
   int n_video_streams;
   int n_audio_streams;
   GstAdapter *adapter;
@@ -109,9 +106,15 @@ struct _GstRMDemux {
   gboolean segment_running;
   gboolean running;
 
+  /* Whether we need to send a newsegment event */
+  gboolean need_newsegment;
+
   /* Current timestamp */
   GstClockTime cur_timestamp;
 
+  /* First timestamp */
+  GstClockTime first_ts;
+
   int n_chunks;
   int chunk_index;
 
